/*
*   _____/\\\\\\\\\________/\\\\\\\\\\\\__/\\\\\\\\\\\\\\\__/\\\\\_____/\\\__/\\\\\\\\\\\\________/\\\\\\\\\_______/\\\\\\\\\_____        
*    ___/\\\\\\\\\\\\\____/\\\//////////__\/\\\///////////__\/\\\\\\___\/\\\_\/\\\////////\\\____/\\\\\\\\\\\\\___/\\\///////\\\___       
*     __/\\\/////////\\\__/\\\_____________\/\\\_____________\/\\\/\\\__\/\\\_\/\\\______\//\\\__/\\\/////////\\\_\///______\//\\\__      
*      _\/\\\_______\/\\\_\/\\\____/\\\\\\\_\/\\\\\\\\\\\_____\/\\\//\\\_\/\\\_\/\\\_______\/\\\_\/\\\_______\/\\\___________/\\\/___     
*       _\/\\\\\\\\\\\\\\\_\/\\\___\/////\\\_\/\\\///////______\/\\\\//\\\\/\\\_\/\\\_______\/\\\_\/\\\\\\\\\\\\\\\________/\\\//_____    
*        _\/\\\/////////\\\_\/\\\_______\/\\\_\/\\\_____________\/\\\_\//\\\/\\\_\/\\\_______\/\\\_\/\\\/////////\\\_____/\\\//________   
*         _\/\\\_______\/\\\_\/\\\_______\/\\\_\/\\\_____________\/\\\__\//\\\\\\_\/\\\_______/\\\__\/\\\_______\/\\\___/\\\/___________  
*          _\/\\\_______\/\\\_\//\\\\\\\\\\\\/__\/\\\\\\\\\\\\\\\_\/\\\___\//\\\\\_\/\\\\\\\\\\\\/___\/\\\_______\/\\\__/\\\\\\\\\\\\\\\_ 
*           _\///________\///___\////////////____\///////////////__\///_____\/////__\////////////_____\///________\///__\///////////////__
*
*                                                       "Maintaining the agenda is our top priority."
*                                                       (C) Innovation Science LLC 2024
*/

var currentWID = 0;
var openWindows = [];
var currentZIndex = 0;

// Load mobile-detect magic js file to instantiate a MobileDetect and return mobile status (null if not mobile).
var mobileStatus = import("./mobile-detect.js").then((md) => {
    var detector = new MobileDetect(window.navigator.userAgent);
    return detector.mobile();
});

/*
 * initAgendaWM starts the window manager.
 */
async function initAgendaWM() {
    // We need to wait for mobileStatus to resolve.
    mobileStatus = await mobileStatus;
    if(!mobileStatus) {
        initAgendaDesktop();
    } else {
        initAgendaMobile();
    }
}

function initAgendaDesktop() {
    console.log("AGENDA2 Desktop");

    var taskbar = createTaskbarHtml();
    var taskbarContainer = document.getElementById("appMenuContainer");
    taskbarContainer.insertAdjacentHTML('beforeend', taskbar);
    
    createWindow("Test App 1", "/temp.html");
    createWindow("Test App 2", "/temp.html");
}

function initAgendaMobile() {
    console.log("AGENDA2 Mobile");

    var taskbar = createTaskbarHtml();
    var taskbarContainer = document.getElementById("appMenuContainer");
    taskbarContainer.insertAdjacentHTML('beforeend', taskbar);

    createWindow("Test App 1", "/temp.html");
    createWindow("Test App 2", "/temp.html");
}

function createTaskbarHtml() {
    if(mobileStatus) {
        return taskbar = createTaskbarHtmlMobile();
    }
    return taskbar = createTaskbarHtmlDesktop();
}

function createTaskbarHtmlDesktop() {
    var taskbar = `
    <div class="dAppMenuContent" id="appMenuContent">
        <a onclick="createWindow('Test', '/temp.html')"><p><img src="/nope.png" class="appMenuImage">Test App</p></a>
        <hr />
        <a target="_top" href="#"><p><img src="/nope.png" class="appMenuImage">Shutdown DremJS</p></a>
    </div>
    `

    return taskbar;
}

function createTaskbarHtmlMobile() {
    var taskbar = `
    <div class="mAppMenuContent" id="appMenuContent">
        <a onclick="createWindow('Test', '/test.html')"><p><img src="/nope.png" class="appMenuImage"> Test App</p></a>
        <hr />
        <a target="_top" href="#"><p><img src="/nope.png" class="appMenuImage"> Shutdown DremJS</p></a>
    </div>
    `
    return taskbar;
}

/*
 * Create window does exactly what it says on the tin.
 * It takes a human-friendly window name, href to the application,
 *  and also whether or not it's draggable or not (default true).
*/
function createWindow(name, href, draggable = true, widget = false) {
    // The window strings are just html. Actually a decent bit lifted from og Agenda, but implemented better obviously.
    var wid = getNewWID();
    var window = createWindowHtml(name, href, draggable, widget);
    var icon = createIconHtml(name, href, draggable, widget);

    console.log(window);
    // Now we can insert the new window text into the applications container. 
    var parent=document.getElementById('applications');
    parent.insertAdjacentHTML('beforeend', window);

    // We don't want to make the window draggable if AGENDA2 is in mobile mode
    //  or the window explicitly said not to be draggable.
    if(draggable && !mobileStatus) {
        makeDraggable(wid);
    }

    // We now set the window to the foreground.
    moveToFront(wid);
}

/*
 * Simple decision making logic. Create a mobile HTML if in mobile mode,
 *  and create a desktop HTML if not.
*/
function createWindowHtml(name, href, draggable, widget) {
    if(mobileStatus) {
        return createWindowHtmlMobile(name, href, draggable, widget);
    }

    return createWindowHtmlDesktop(name, href, draggable, widget);
}

function createWindowHtmlDesktop(name, href, draggable, widget) {
    var window = `
    <div onmousedown="moveToFront(${wid});"
        name="${name}"
        class="dAppWindow"
        id="window-${wid}">
        <div class="dAppTask" id="bar-${wid}">
            <div class="dAppTaskChild">
                <input type="button" onclick="destroyWindow(${wid})" value="X" />
                <input type="button" onclick="maximizeWindow(${wid})" value="\u25A1" />
                <input type="button" onclick="minimizeWindow(${wid})" value="_" />
            </div>
            <div class="dAppTaskChild" id="text-${wid}">
                <p>${name}</p>
            </div>
        </div>
        <br />
        <div class="dAppFrame">
            <iframe name="app-${wid}" id="app-${wid}" src="${href}"></iframe>
        </div>
    </div>
    `

    return window;
}

function createWindowHtmlMobile(name, href, draggable, widget) {
    var window = `
    <div onmousedown="moveToFront(${wid});"
        name="${name}"
        class="mAppWindow"
        id="window-${wid}">
        <div class="mAppTask" id="bar-${wid}">
            <div class="mAppTaskChild">
                <input type="button" onclick="destroyWindow(${wid})" value="X" />
            </div>
            <div class="mAppTaskChild" id="text-${wid}">
                <p>${name}</p>
            </div>
        </div>
        <br />
        <div class="mAppFrame">
            <iframe name="app-${wid}" id="app-${wid}" src="${href}"></iframe>
        </div>
    </div>
    `

    return window;
}

function createIconHtml(name, href, draggable, widget) {
    
}

/*
 * Destroy the window by sending it's element to the shadow realm.
 */
function destroyWindow(wid) {
    var window = document.getElementById(`window-${wid}`);
    var windowContainer = document.getElementById('applications');

    windowContainer.removeChild(window);
}

/*
 * I went about moving windows to the front differently than in the original version of Agenda.
 * A simple counter is used, and when the function is used, it sets the z-index to an incrementing
 *  counter. It's incremented after setting the z-index, so that the next time it's run it'll be sure
 *  to set it to the foreground. Did that make sense? I'm so tired.
 *
 * Obviously, this is how this should be done, unlike the bodge job before.
*/
function moveToFront(wid) {
    var window = document.getElementById(`window-${wid}`);

    window.style.zIndex = `${currentZIndex}`;
    currentZIndex++;
}

function maximizeWindow(wid) {
    console.log("maximizeWindow not yet implemented");
}

function minimizeWindow(wid) {
    console.log("minimizeWindow not yet implemented");
}

// Actual magic chicanery which I'm pretty sure I did last time lmao
function makeDraggable(wid) {
    var win = document.getElementById(`window-${wid}`);
    var bar = document.getElementById(`bar-${wid}`);
    var barText = document.getElementById(`text-${wid}`);

    win.addEventListener('mousedown', function(e) {
        // This bodge detects whether the object that was clicked
        if(e.target != bar && e.target.parentNode != barText) {
            reset();
            return;
        }
        e.preventDefault();
        var offsetX = e.clientX - parseInt(window.getComputedStyle(this).left);
        var offsetY = e.clientY - parseInt(window.getComputedStyle(this).top);

        function mouseMoveHandler(e) {
            var top = (e.clientY - offsetY);
            if(top < 51) {
                top = 51;
            }
            win.style.top = top + 'px';
            win.style.left = (e.clientX - offsetX) + 'px';
        }

        function reset() {
            window.removeEventListener('mousemove', mouseMoveHandler);
            window.removeEventListener('mouseup', reset);
        }

        window.addEventListener('mousemove', mouseMoveHandler);
        window.addEventListener('mouseup', reset);
    });
}

/*
 * Return a new window id and increment it.
 * returns wid: current window id for use in a new window
 */
function getNewWID() {
    wid = currentWID;
    currentWID+=1;
    return wid;
}

function toggleAppMenu() {
    var appMenu = document.getElementById("appMenuContent");

    if(appMenu.style.visibility == "hidden") {
        appMenu.style.visibility = "visible";
    } else {
        appMenu.style.visibility = "hidden";
    }
}

initAgendaWM();